/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 26 February 2003, 19:46
 */

package com.exponto.jstock;
import org.apache.log4j.Logger;
/**
 * ChartData.java
 *
 * Load the dataset from file or other source (to be implemented). 
 * Can transform dataset to other formats (log, weekly) as required 
 *
 * @author  kaare
 */
public class ChartData {
	static Logger logger = Logger.getLogger(ChartData.class);
    private String ticker;
    private String path = "";
    private String extension = "";
//    private DataSeries dataSeries = new DataSeries();
    private LogDataSeries logDataSeries = new LogDataSeries();
	private LogDataSeries logWeeklyDataSeries;
 
    /** Creates a new instance of ChartData */
    public ChartData() {
    }

    /**
     * Creates a new instance of ChartData
     * 
     * @param str  representing the ticker */    
    public ChartData(String str) {
        ticker = str;
    }
    
	/**
	 * Reads dataseries from a source, currently only files in "netfonds" format are understood
	 *
	 * @param source Source (e.g. path and filename) of the dataseries*/ 
    
	public void loadLogDataSeries(String source, int numberToGet) throws TickerNotFoundException {
		// To read from other sources (e.g. database or internet), implement a 
		// new private method and call that instead
		logDataSeries = readFile(source, numberToGet);
		logDataSeries.rotate();
//		if (Util.validate(logDataSeries)) {
//			System.out.println("ChartData - File " + source + " seems to be fairly correct");	
//		}
	}
  
/**
 * 
 * 
 * @return the LogDataSeries
 */  
	public LogDataSeries getLogDataSeries() {
		return logDataSeries;
	}
   
    /**
     * 
     * @param path The file path
     * @param extension of the files  
     * */    
    public void setPathAndExtension(String path, String extension) {
        this.path = path;
        this.extension = extension;
    }

/**
 * Returns a LogSeriesData that contains weekly data. If that dosn't
 * exist, it is computed when this methode is called
 * 
 * @return LogDataSeries with weekly data.
 */
	public LogDataSeries getWeeklyLogDataSeries() {
		if (logWeeklyDataSeries == null) {
			loadWeeklyLogDataSeries();			
		}
		return logWeeklyDataSeries;
	}
	
	private void loadWeeklyLogDataSeries() {
		logWeeklyDataSeries = new LogDataSeries(true);
		int index = 0;
		double log10 =  Math.log(10);
		LogTradingData tempLTD;
		LogTradingData lTDWeek;
		logDataSeries.setCurrentPeriode();
		tempLTD = logDataSeries.getNextLTD();
		if (tempLTD == null) {
			logger.error("TempLTD = null");
			logger.error(logDataSeries.debugInfo());
		}
		int currentWeek;
		boolean hasMore = true;
		while (hasMore) { 
		//(logDataSeries.hasNextLTD()) {
			lTDWeek = new LogTradingData();
			lTDWeek.low = Constants.lowStart;
			currentWeek = tempLTD.day.week;
			lTDWeek.day = tempLTD.day;
			while (currentWeek == tempLTD.day.week) {
				if (tempLTD.isTraded()) {
					lTDWeek.status = tempLTD.status;
					// Find first valid for open
					if (lTDWeek.open == 0.0) {
						lTDWeek.open = tempLTD.open;
					}
					lTDWeek.volume = lTDWeek.volume + tempLTD.volume;
					// Close is always latest valid value
					lTDWeek.close = tempLTD.close;
					if (tempLTD.low < lTDWeek.low) {
						lTDWeek.low = tempLTD.low;
						lTDWeek.logLow = tempLTD.logLow;
					}
					if (tempLTD.high > lTDWeek.high) {
						lTDWeek.high = tempLTD.high;
						lTDWeek.logHigh = tempLTD.logHigh;
					}
					if (lTDWeek.open > lTDWeek.close) { 
						lTDWeek.logX = Math.log(lTDWeek.open)/log10;
						lTDWeek.logDiff = lTDWeek.logX - Math.log(lTDWeek.close)/log10;
					} else {
						lTDWeek.logX = Math.log(lTDWeek.close)/log10;
						lTDWeek.logDiff = lTDWeek.logX - Math.log(lTDWeek.open)/log10;
					}
				}
				// Only read if there is more data in the series
				if (logDataSeries.hasNextLTD()) {
					tempLTD = logDataSeries.getNextLTD();
//					isStored = false;
				} else {
					// Force exit of while loop
					currentWeek = -1;
					hasMore = false;
				} 
			}
			logWeeklyDataSeries.storeResultWeekly(lTDWeek);
//			System.out.println("Weekly data - Week no: " + currentWeek +" High: " + lTDWeek.high + " Low: " + lTDWeek.low);			
		}
		logWeeklyDataSeries.setTickerId(logDataSeries.getTickerId());
	}
//  
//    public void loadRsiSeries() {
//        // Add code
//    }
    
    /**
     * Reads a file a converts it to a LogDataSeries. 
     * 
     * @param str is the complete filename of the file to be read
     * @param numberToGet determines how many days that is read from the file
     * @return  a LogDataSeries generated from data */    
    private LogDataSeries readFile(String str, int numberToGet) throws TickerNotFoundException {
        LogDataSeries lDS;
        ReadNetfondsFile rNF = new ReadNetfondsFile();
       // try {
			lDS = rNF.readSeries(str, numberToGet);
        return lDS;
    }
	/**
	 * Returns the TickerID of the LogdataSeries associated with this class.
	 * 
	 * @return the TickerId 
	 */
	public TickerID getTickerId() {
		return logDataSeries.getTickerId();
	}

    
//	/**
//	 * Believe it isn't used anymore
//	 * @return the ticker
//	 */
//	public String getTicker() {
//		return ticker;
//	}

//	/**
//	 * Believe it isn't used anymore
//	 * @param string the ticker
//	 */
//	public void setTicker(String string) {
//		ticker = string;
//	}


}
