/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 29-Sep-2003
 *  Assistance wanted - This class can be simplified and optimized a lot. For the time being I'm
 * 	only trying to make it work... 
 */
package com.exponto.jstock;
import java.sql.*;
import java.util.HashMap;
import java.util.ArrayList;
import org.apache.log4j.Logger;

/**
 * Creates and ecexute all sql statements.
 * 
 * In order to reduce queries to the db, a cople of the smaller key tables are kept in
 * memmory   
 * 
 * @author kaare
 *
 */
public class JstockDB {
	static Logger logger = Logger.getLogger(JstockDB.class);
	String exchangeName = "ose";
	String tickerName;
	Integer exchangeId;
	int tickerId;
	// The combination of exchangeid and tickerId is used as unique index in most other tables, stored here to
	// reduce # of db lookups
	HashMap exchangeList;
	HashMap [] tickerList;
	// Used to map exchangeId to selector for the right HashMap in the tickerList[]
// Why Hashmap: The solution may seem more complex than needed  
// (i.e. a lot of int -> Integer -> int), but as I don't now the number of rows 
// returned from DB, to choice is to first go through the result and then 
// make int []'s, or to use ArrayList, HashMap or other "growable" clases.
// As static int [] must be resized if a new exchange is added they too would
// be "ugly". HashMaps are supposed to be fast to search.   
	HashMap idMapper;
	static JstockDB instance;

	private JstockDB() {
		initHashMaps();
	}
	static public JstockDB getInstance() {
		if (instance==null) {
			instance = new JstockDB();
		}
		return instance;
	}
	
	public ArrayList loadWatchList() {
		// Joins tables and selects tickerid, ticker and exchangename from all 
		// tickerids in the watchlist
		String sql = "SELECT w.tickerid, t.ticker, e.exchangename FROM tickers t, watchlist w, exchange e WHERE (t.index = w.tickerid) AND (t.exchangeid = e.index);";
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		int ticker;
		Integer value;
		String tickerName;
		String exchangeName;
		ArrayList result = new ArrayList();
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql);
			while (sqlResult.next()) {
				ticker = sqlResult.getInt(1);
				tickerName = sqlResult.getString(2);
				exchangeName = sqlResult.getString(3);
				value = new Integer(ticker);
				TickerID t = new TickerID(ticker, tickerName, exchangeName);
				result.add(t);							
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not load watchlist from db", e);
			//System.out.println("Class DBManager: Could not load watchlist from db");
			//e.printStackTrace();
			return null;
		}
		return result;
	}
	
	public String toggleWatchList(TickerID tId) {
		tickerId = this.getTickerId(tId.getTickerName(),  tId.getExchangeName());
//		tickerId = this.getTickerId(ticker, exchange);
// Done the brut force way, must check if tickerId already is in DB 
		String sql = "SELECT tickerid FROM watchlist WHERE (tickerid = " + tickerId + ");";
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		int t = -1;
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql);
			while (sqlResult.next()) {
				t = sqlResult.getInt(1);
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not check watchlist from db", e);
//			System.out.println("Class DBManager: Could not check watchlist from db");
//			e.printStackTrace();
		}
		sql = null;
		if (t < 0) {
			// ticerId is not in watchlist
			sql = "INSERT INTO watchlist (tickerid) VALUES (" + tickerId + ")";	
		} else {
			sql = "DELETE FROM watchlist WHERE tickerid = " + tickerId;
		}
		int rowCount;
		String str;
		rowCount = DBManager.executeUpdate(sql);
		if (rowCount > 0) {
			str = " Watch list is updated";	 
		} else {
			str = " Problems updating watch list";
		}
		logger.info(str);
		return str;
	}
	
	public String loadNotes(TickerID tId) {
		tickerId = this.getTickerId(tId.getTickerName(),  tId.getExchangeName());
		String str = null;
		int rowExists = -1;
		//tickerName = ticker;
		//tickerId = this.getTickerId(ticker, exchange);
//		If the index exists, a new row is not created 
		String sql = "SELECT note, index FROM notes WHERE tickerid = "  + tickerId;
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql);
//			while (sqlResult.next()) {
			if (sqlResult.next()) {
				str = sqlResult.getString(1);
				rowExists = sqlResult.getInt(2);
			}
			if (rowExists == -1) {
//				Create an empty note row in DB	
				this.createNote(tickerId);
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not load notes for " + tId.getTickerName(), e);
			//System.out.println("Class DBManager: Could not load notes for " + ticker);
			//e.printStackTrace();
			return null;
		}
		return str;
	}

	public String saveNotes(TickerID tId, String text) {
		tickerId = this.getTickerId(tId.getTickerName(),  tId.getExchangeName());
		String sql = "UPDATE notes SET note = '" + text + "'" +
			" WHERE tickerid = "  + tickerId;
		int rowCount;
		String str;
		rowCount = DBManager.executeUpdate(sql);
		if (rowCount > 0) {
			str = " Notes saved for " + tickerId;	 
		} else {
			str = " Problems saving notes for " + tickerId;
		}
		logger.info(str);
		return str;
	}
	/**
	 * Loads lines from the database
	 * 
	 * Discards any lines that may be in the array list before loading.
	 * 
	 * @param l
	 * @return the same Lines with content fresh from database.
	 */
	public Lines loadLines(Lines l) {
		int rowExists = -1;
		tickerId = this.getTickerId(l.getTickerId().tickerName,  l.getTickerId().getExchangeName());
		String sql = "SELECT index, datex1, valuey1, datex2, valuey2 FROM lines WHERE tickerid = "  + tickerId + "AND type = " + l.getType();
		l.persistentLines.clear();
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql);
			while (sqlResult.next()) {
//			if (sqlResult.next()) {
				PersistentLine pL = new PersistentLine(sqlResult.getInt(1), sqlResult.getString(2), sqlResult.getDouble(3), sqlResult.getString(4), sqlResult.getDouble(5) );
				l.addLine(pL);
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not load lines for " + (l.getTickerId().tickerName), e);
			//System.out.println("Class DBManager: Could not load lines for " + (l.getTickerId().tickerName));
			//e.printStackTrace();
			return null;
		}
		return l;
	}

	/**
	 * Saves any modification to the lines in the database. If there are any
	 * new lines, the lines will be (re)read from the database.
	 * 
	 * @param l the lines to save
	 * @return The lines as they would be if loaded from database
	 */
	 
	public ArrayList saveLines(Lines l) {
		int tickerId = this.getTickerId(l.getTickerId().tickerName,  l.getTickerId().getExchangeName());
		int type = l.getType();
		boolean reload = false;
		Lines newLines = l;
		l.reset();
		PersistentLine pL = l.hasNext();
		while (pL != null) {
			String sql = null;
			String s1 = pL.getX1().toDigitString();
			String s2 = pL.getX2().toDigitString(); 
			int rowCount;
			if (pL.getLineIndex() >= 0) {
				// Line exists in database
				if (pL.isDeleted()) {
					System.out.println("Deleting line");
					sql = "DELETE FROM lines WHERE index = " +pL.getLineIndex();	
				} else {
					sql = "UPDATE lines SET datex1 = '" + s1 + "', valuey1 = " + pL.getY1() +", datex2 = '" + s2 + "', valuey2 = " + pL.getY2() + "WHERE index = " +pL.getLineIndex();
				}
			} else {
				// New Line
				logger.debug("Here is a new line");
				reload = true;
				if (!pL.isDeleted()) {
					// Only insert if not deleted
					sql = "INSERT INTO lines(tickerid, type,  datex1, valuey1, datex2, valuey2) VALUES (" + tickerId + "," + type + ",'" + s1 + "'," + pL.getY1() +", '" + s2 + "'," + pL.getY2()+ ")";
				}
			}
			if (sql!=null) {
				rowCount = DBManager.executeUpdate(sql);
			} 
			pL = l.hasNext();
		}
		if(reload) {
			newLines = this.loadLines(newLines);
			logger.debug("Lines reloaded from database");
		}
		return newLines.persistentLines;		
	}
	
public String removePersistentLine(PersistentLine pL) {
	String str = null;
	if (pL != null) {
		String sql;
		String s1 = pL.getX1().toDigitString();
		String s2 = pL.getX2().toDigitString(); 
		int rowCount;
		if (pL.getLineIndex() >= 0) {
			// Line exists in database
			sql = "DELETE FROM lines WHERE index = " +pL.getLineIndex();
			rowCount = DBManager.executeUpdate(sql);
			if (rowCount > 0) {
				str = " Line removed";	 
			} else {
				str = " Problems removing line";
			}
		}
	}
	logger.info(str);
	return str;
}	
	
	public int createTicker(String ticker, String exchange) {
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		int tickerListIndex = 0;
		int rowCount = 0;
		// Check if exchange already defined
		exchangeId = (Integer) exchangeList.get(exchange);
		if (exchangeId == null) {
			// If not create it
			exchangeId = this.createExchange(exchange);
		}
		tickerListIndex = this.getTickerListNumber(exchangeId);
		// String for adding the ticker to the tickers table
		String sqlTicker = "INSERT INTO tickers (exchangeid,  ticker) VALUES (" + exchangeId.intValue() + ",  '" + ticker.toLowerCase() + "')";
		rowCount = DBManager.executeUpdate(sqlTicker);
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			String sqlTickerIndex = "SELECT index FROM tickers WHERE exchangeid= '" + exchangeId.intValue() + "' AND ticker = '" + ticker +"'";
			sqlResult = sqlStmnt.executeQuery(sqlTickerIndex);
			int i = 0;
			// Only one row should be returned here as ticker and exchangename is unique
			while (sqlResult.next()) {
				i = 1;
				tickerId = sqlResult.getInt(i++);
				Integer value = new Integer(tickerId);
				// Need to increase tickerList array if a new exchange was added
				if (tickerList[tickerListIndex] == null) {
					HashMap [] tempMap = new HashMap [exchangeList.size()];
					for (i = 0; i<exchangeList.size() - 1; i++) {
						// Copy existing tickerlist to tempmap 
						tempMap[i] = tickerList[i];
					}
					// Add the new one
					tempMap[exchangeList.size()-1] = new HashMap(32);
					//Update tickerList with the new hashmap
					tickerList = tempMap;
				}	
//				Update hashmap			
				tickerList[exchangeList.size() - 1].put(ticker, value);
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not create new ticker", e);
			//System.out.println("Class DBManager: Could not create new ticker");
			//e.printStackTrace();
			return 0;
		}
		return rowCount;
	}

	private int createNote(int tickerId) {
		// Use this when sure that a row for tickerId doesn't exist (e.g. loadNote failed)
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		int rowCount = 0;
		//		String for adding the tickerid  in table notes
		String sqlNotes = "INSERT INTO notes (tickerid) VALUES (" + tickerId + ")";
   		rowCount = DBManager.executeUpdate(sqlNotes);
//		try {
//			Connection con = dBMan.getConnection();
//			sqlStmnt = con.createStatement();	
//			rowCount = sqlStmnt.executeUpdate(sqlNotes);
//		} catch (Exception e) {
//			System.out.println("Class DBManager: Could not create new note");
//			e.printStackTrace();
//		}
		logger.debug("createNote Rowcount: " + rowCount);
		//System.out.println("createNote Rowcount: " + rowCount);
		return rowCount;
		
	}
	
	private Integer createExchange(String exchange) {
		ResultSet sqlResult;// = null;
		Statement sqlStmnt = null;
		DBManager dBMan = DBManager.getInstance();
		int rowCount;
		exchangeId = null;
//		String for adding exchangename in table exchange
		String sqlExchange = "INSERT INTO exchange (exchangename) VALUES ('" + exchange.toLowerCase() + "')";
//		String for getting the index of the new exchangename				
		String sqlExIndex = "SELECT index FROM exchange WHERE exchangename = '" + exchange +"'";
		// If not update and get id
		rowCount = DBManager.executeUpdate(sqlExchange);
		try {
//			rowCount = sqlStmnt.executeUpdate(sqlExchange);
			sqlResult = sqlStmnt.executeQuery(sqlExIndex);
			int i = 0;
			// Only one row should be returned here as exchangename is unique
			while (sqlResult.next()) {
				i = 1;
				exchangeId = new Integer(sqlResult.getInt(i++));
				//Update hashmap
				exchangeList.put(exchange, exchangeId);
				Integer id = new Integer(exchangeList.size()-1);
				idMapper.put(exchangeId, id);
			}
		} catch (Exception e) {
			logger.error("Class DBManager: Could not create new exchange", e);
			//System.out.println("Class DBManager: Could not create new exchange");
			//e.printStackTrace();
		}
		return exchangeId;
	}

	private int getTickerId(String ticker, String exchange) {
		Integer temp;
		temp = (Integer) exchangeList.get(exchange);
		if (temp==null) {
			temp = createExchange(exchange);	
		}
		int i = this.getTickerListNumber(temp);
		temp = (Integer) tickerList[i].get(ticker);
		if (temp == null) {
			createTicker(ticker, exchange);
			temp = (Integer) tickerList[i].get(tickerName);
		}
		return temp.intValue();
	}

	private int getTickerListNumber(Integer exchangeId) {		
		Integer temp = (Integer) idMapper.get(exchangeId);
		return temp.intValue();
	}
	private int getTickerListNumber(int exchangeId) {	
		Integer temp = new Integer(exchangeId); 
		temp = (Integer) idMapper.get(temp);
		return temp.intValue();
	}
	
	private void initHashMaps() {
		String sql = "SELECT  * FROM exchange";
		DBManager dBMan = DBManager.getInstance();
		ResultSet sqlResult = null;
		Statement sqlStmnt = null;
		exchangeList = new HashMap();
		idMapper = new HashMap();
		int index;
		try {
			Connection con = dBMan.getConnection();
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql); 
			int i = 1;
			int j = 0;
			while (sqlResult.next()) {
				i = 1;
				exchangeId = new Integer(sqlResult.getInt(i++));
				exchangeName = sqlResult.getString(i);
				//Integer value = new Integer(index);
				Integer number = new Integer(j++);
				exchangeList.put(exchangeName, exchangeId);
				idMapper.put(exchangeId, number);
				logger.debug("Index = " + exchangeId + " Exchange = " + exchangeName);
				//System.out.println("Class DBManager: Index = " + exchangeId + " Exchange = " + exchangeName);
			} 
			// Creating on ticker hashmap per exchangeId
			tickerList = new HashMap [exchangeList.size()];
			for (i = 0; i<exchangeList.size(); i++) {
				tickerList[i] = new HashMap(32);
			}
			sql = "SELECT  * FROM tickers";
			int exchangeNo;
			sqlStmnt = con.createStatement();
			sqlResult = sqlStmnt.executeQuery(sql);
			while (sqlResult.next()) {
				i = 1;
				index = sqlResult.getInt(i++);
				exchangeNo = sqlResult.getInt(i++);
				tickerName = sqlResult.getString(i++);
				Integer value = new Integer(index);
				tickerList[this.getTickerListNumber(exchangeNo)].put(tickerName, value);
				//logger.debug("Index = " + index + " Ticker = " + tickerName + " E-id = " + exchangeNo);
				//System.out.println("Class DBManager: Index = " + index + " Ticker = " + tickerName + " E-id = " + exchangeNo);
			}
		} catch (Exception e) {
			logger.error("Class JstockDB: SQL error in initHashMaps()", e);
			//System.out.println("Class JstockDB: SQL error in initHashMaps()");
			//System.out.println(e);
			//e.printStackTrace();
		}

	}
}

