/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 28-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;

/**
 * @author kaare
 *
 */
public interface IndicatorParametersInterface {
	
	/**
	 * Checks if the parameters settings of this indicator indicates that the
	 * Indicator is activated (e.g. number of days used in RSI calcultaion set 
	 * to a value larger than 0).
	 * 
	 * @return true this indicator shall be drawn (based on the 
	 * parameters setting). False otherwise 
	 */
	public abstract boolean drawThis();
	/**
	 * Provides the Options menu with components to use for this indicator.
	 * The JComponents are responsible for notifying "their" indicator when 
	 * values are changed, and also update the properties files.
	 * 
	 * Note that the receiving class makes a few assumptions on the 
	 * content of the array.
	 * 
	 * @return an JComponent array with the components that shall be 
	 * displayed
	 */
	public abstract JComponent[][] getUserInterface();
	
	/**
	 * Used to update the parameters value within the IndicatorParameter 
	 * class. 
	 */
	public abstract void updateParameters();

}