/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 17. okt. 2003
 */
package com.exponto.jstock.popupwindows;

import javax.swing.*;

import com.exponto.jstock.UserPreferences;

import java.awt.*;
import java.awt.event.*;


/**
 * Shows a JFrame with several tabs for changing various options.
 * 
 * @author kaare
 */
public class PreferenceEditor extends JFrame {
	private JTabbedPane tabbedPane;
	JPanel controlPanel;
	ScrollPane about;
	JScrollPane rulesPane;
	JPanel aboutPanel;
	
	//IndicatorsMenuOld iM;
	IndicatorsMenu iM;
	RulesMenu rM;
	FileInputMenu fM;
	DatabaseMenu dbM;
	SystemMenu sM;
	
// Control		
	private JButton restore;
	private JButton save;
	private JButton quit;
	private JButton quitNoSave;

	private static UserPreferences userPref;	
	public PreferenceEditor() {
		super("PreferenceEditor");
		userPref = UserPreferences.getInstance();
		tabbedPane = new JTabbedPane();
//		getContentPane().add(mainPanel, BorderLayout.CENTER);
		controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout());
		about = new ScrollPane();
		aboutPanel = new JPanel();
		aboutPanel.add(About.getLicense());
		about.add(aboutPanel);
		makeControlPanel();
		addTabs();
	}
	
	private void addTabs() {
// TODO: Bug - Panels (db & file input) not properly redrawn if checkbox is toggeled and no scrollbars in the scrollpane
//		iM = new IndicatorsMenuOld();
		iM = new IndicatorsMenu();
		tabbedPane.addTab("Indicators", null, iM.getIndicatorsPanel(), "Modify parameters for the technical indicators");

		rM = new RulesMenu();
		rulesPane = rM.getRulesPane();
		tabbedPane.addTab("Rules", null, rulesPane, "Modify rules for automatic generation of buy or sell signal");
		
		ScrollPane dbS = new ScrollPane();
		dbM = new DatabaseMenu();
		dbS.add(dbM.getPanel());
		tabbedPane.addTab("Database", null, dbS, "Parameters for database connectivity");
		ScrollPane sS = new ScrollPane();
		sM = new SystemMenu();
		sS.add(sM.getPanel());
		tabbedPane.addTab("System", null, sS, "A few system parameters");
		ScrollPane fS = new ScrollPane();
		
		fM = new FileInputMenu();
		fS.add(fM.getPanel());
		tabbedPane.addTab("File input", null, fS, "Modify colum numbers for quote data read from file");

		tabbedPane.addTab("About", null, about, null);

		getContentPane().add(controlPanel, BorderLayout.SOUTH); 
		getContentPane().add(tabbedPane, BorderLayout.CENTER);
		pack();
		setVisible(true);

	}
	
	private void makeControlPanel() {
		restore = new JButton("Undo Changes");
		restore.setToolTipText("Restores parameters for ALL tabs");
		save = new JButton("Save");
		save.setToolTipText("Saves parameters for ALL tabs");
		quit = new JButton("Close and save");
		quit.setToolTipText("Saves parameters for ALL tabs and close this window");
		quitNoSave = new JButton("Quit without save");
		quit.setToolTipText("Quits this window without saving the parameters");
		controlPanel.add(restore);
		controlPanel.add(save);
		controlPanel.add(quit);
		controlPanel.add(quitNoSave);

		restore.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				restore();
			}
		});		
		save.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				save();
			}
		});
		quit.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				save();
				quit();
			}
		});
		quitNoSave.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				quit();
			}
		});
	}
	
	private void restore() {
		iM.restore();
		fM.restore();
		dbM.restore();
		sM.restore();
	}
	private void save() {
		iM.save();
		fM.save();
		dbM.save();
		sM.save();
	}

	private void quit() {
		dispose(); 
	}
}
